import java.util.*;

public class Ball extends PhysicsElement  {
   private static int id=0;  // Ball identification number
   private final float mass;
   private final float radius;
   private double crossSection; // to compute air friction 
   private double pos_t;     // current position at time t
   private double pos_tPlusDelta;  // next position at t + delta
   private double speed_t;   // speed at time t
   private double speed_tPlusDelta;   // speed at t + delta
   private double a_t;    // acceleration at time t
   private double a_tMinusDelta;  // acceleration at t-delta
   
   private Ball(){   // nobody can create a ball without state
     super(id++);
     mass=0;
     radius=0;
   }
   public Ball(float mass, float r, float position, float speed, MyWorld world){
      super(id++);
      this.mass = mass;
      radius = r;
      pos_t = position;
      speed_t = speed;
      a_t = 0;
      a_tMinusDelta = getNetForce(world.getGravity(),world.getViscosity())/mass;
      crossSection = r;
      world.addElement(this);
   }
   public float getMass() {
      return mass;
   }
   public float getRadius() {
      return radius;
   }
   public double getPosition() {
      return pos_t;
   }
   public double getSpeed() {
      return speed_t;
   }
   public boolean collide(PhysicsElement pe){
      return Math.abs(pe.getPosition()-pos_t)<(pe.getRadius()+radius);
   }
   private double getNetForce(float g,float viscosity) {
     double force;
     force = mass*g; /*plus because y axis grows downwards */
     force-= speed_t*viscosity*crossSection; 
     return(force);    
   }
   public void computeNextState(double delta_t, MyWorld w) {
     float g=w.getGravity();
     float viscosity=w.getViscosity();
     double speed;
     PhysicsElement pe;
     if ((pe=w.getNear(this))!= null){ /* elastic collision */
        speed=(speed_t*(mass-pe.getMass())+2*pe.getMass()*pe.getSpeed())/(mass+pe.getMass());
        a_tMinusDelta = getNetForce(g, viscosity)/mass;
     } else 
         speed = speed_t;
    
     a_t = getNetForce(g, viscosity)/mass;
     speed_tPlusDelta = speed + (3*a_t - a_tMinusDelta)*delta_t/2;
     pos_tPlusDelta = pos_t + speed*delta_t + (4*a_t-a_tMinusDelta)*delta_t*delta_t/6;
   }
   public void updateState(){
     pos_t = pos_tPlusDelta;
     speed_t = speed_tPlusDelta;
     a_tMinusDelta = a_t;
   }
   public String getDescription() {
      return "Ball_" + getId()+": x ";
   }
   public String getState() {
      return getPosition()+" ";
   }
}
